@@ -2,6 +2,7 @@
 """
 Tasks used to compile treasury measures
 """
+import datetime
 import transaction
 
 from pyramid_celery import celery_app
@@ -36,6 +37,10 @@ from endi_celery.tasks import utils
 logger = utils.get_logger(__name__)
 
 
+class KnownError(Exception):
+    pass
+
+
 class GridCacheItem:
     """
     Wraps a grid in cache and provide storages for
@@ -158,8 +163,8 @@ class BaseMeasureCompiler(object):
         - totals by category
         - totals by type
 
-    Totals are used in a second time to compute "total" measures (not related to
-    account numbers)
+    Totals are used in a second time to compute "total" measures 
+    (not related to account numbers)
     """
     measure_type_class = None
     measure_type_category_class = None
@@ -334,7 +339,7 @@ class BaseMeasureCompiler(object):
         :returns: A list of GridCacheItem
         """
         logger.debug(
-            u"    + Processing datas with {}".format(self.__class__.__name__)
+            "    + Processing datas with {}".format(self.__class__.__name__)
         )
         for operation in self.operations:
             if operation.company_id is None:
@@ -372,7 +377,7 @@ class BaseMeasureCompiler(object):
         Insert common measures in the grids if not set yet
         """
         for grid_item in self.processed_grids:
-            logger.debug("Completing grid %s" % grid_item.grid.id)
+            logger.debug("Completing grid {}".format(grid_item.grid.id))
             measures = grid_item.get_measures()
             for measure_type in self.common_measure_types:
                 measure = measures.get(measure_type.id)
@@ -383,7 +388,8 @@ class BaseMeasureCompiler(object):
                     )
                     grid_item.store_measure(measure)
                 else:
-                    logger.debug("Updating the order of the given measure %s : was %s becomes %s" % (measure.label, measure.order, measure_type.order))
+                    logger.debug("Updating the order of the given measure {} :\
+ was {} becomes {}".format(measure.label, measure.order, measure_type.order))
                     measure.order = measure_type.order
                     self.session.merge(measure)
 
@@ -434,10 +440,10 @@ class TreasuryMeasureCompiler(BaseMeasureCompiler):
     measure_type_category_class = TreasuryMeasureTypeCategory
     measure_grid_class = TreasuryMeasureGrid
     measure_class = TreasuryMeasure
-    label = u"Génération des états de trésorerie"
+    label = "Génération des états de trésorerie"
 
     def get_message(self, grids):
-        return u"""Génération des états de trésorerie
+        return """Génération des états de trésorerie
 
 États de trésorerie générés : {}
 
@@ -502,10 +508,10 @@ class IncomeStatementMeasureCompiler(BaseMeasureCompiler):
     measure_type_category_class = IncomeStatementMeasureTypeCategory
     measure_grid_class = IncomeStatementMeasureGrid
     measure_class = IncomeStatementMeasure
-    label = u"Génération des comptes de résultat"
+    label = "Génération des comptes de résultat"
 
     def get_message(self, grids):
-        return u"""Génération des comptes de résultat :
+        return """Génération des comptes de résultat :
 
 Comptes de résultat traités : {}
 
@@ -517,7 +523,11 @@ enDI
         """
         Build a cache key based on the given operation object
         """
-        return (operation.company_id, operation.date.year, operation.date.month)
+        return (
+            operation.company_id,
+            operation.date.year,
+            operation.date.month
+        )
 
     def get_cache_key_from_grid(self, grid):
         """
@@ -562,6 +572,35 @@ enDI
         for measure in old_measures:
             self.session.delete(measure)
 
+    def _process_common_measures(self):
+        """
+        Add specific process for IncomeStatementMeasureGrid, for now
+        setting the updated_at attribute
+        """
+        logger.debug(
+            u"  + Processing specific datas for child {}".format(self.__class__.__name__)
+        )
+
+        # Calling parent to really process measures
+        logger.debug("Calling parent process common measures")
+        BaseMeasureCompiler._process_common_measures(self)
+
+        # Choosing which date has to be used
+        date_to_set = None
+        if(self.upload.filetype ==
+            AccountingOperationUpload.SYNCHRONIZED_ACCOUNTING):
+            date_to_set = self.upload.updated_at
+        else:
+            date_to_set = self.upload.created_at
+
+        # Setting the date in the non void grids
+        for grid in self.processed_grids:
+            if grid is not None :
+                grid.grid.updated_at = date_to_set
+                self.session.merge(grid.grid)
+
+        self.session.flush()
+        return self.processed_grids
 
 def get_measure_compilers(data_type, grid_type=None):
     """
@@ -585,8 +624,9 @@ def get_measure_compilers(data_type, grid_type=None):
         else:
             return [TreasuryMeasureCompiler, IncomeStatementMeasureCompiler]
 
-MAIL_ERROR_SUBJECT = u"[Erreur] {}"
-MAIL_SUCCESS_SUBJECT = u"[Succès] {}"
+
+MAIL_ERROR_SUBJECT = "[Erreur] {}"
+MAIL_SUCCESS_SUBJECT = "[Succès] {}"
 
 
 def send_success(request, mail_addresses, type_label, message):
@@ -602,7 +642,7 @@ def send_success(request, mail_addresses, type_label, message):
             subject = MAIL_SUCCESS_SUBJECT.format(type_label)
             send_mail(request, mail_addresses, message, subject)
         except:
-            logger.exception(u"Error sending email")
+            logger.exception("Error sending email")
 
 
 def send_error(request, mail_addresses, type_label, message):
@@ -618,7 +658,7 @@ def send_error(request, mail_addresses, type_label, message):
             subject = MAIL_ERROR_SUBJECT.format(type_label)
             send_mail(request, mail_addresses, message, subject)
         except:
-            logger.exception(u"Error sending email")
+            logger.exception("Error sending email")
 
 
 @celery_app.task(bind=True)
@@ -627,17 +667,17 @@ def compile_measures_task(self, upload_id, grid_type=None):
     Celery task handling measures compilation
     """
     logger.info(
-        u"Launching the compile measure task for upload {0}".format(upload_id)
+        "Launching the compile measure task for upload {0}".format(upload_id)
     )
 
     if grid_type is not None:
-        logger.info(u"Only compiling {} grids".format(grid_type))
+        logger.info("Only compiling {} grids".format(grid_type))
 
     transaction.begin()
     upload = AccountingOperationUpload.get(upload_id)
     if upload is None:
         raise Exception(
-            u"No AccountingOperationUpload instance with id {}".format(
+            "No AccountingOperationUpload instance with id {}".format(
                 upload_id
             )
         )
@@ -654,7 +694,7 @@ def compile_measures_task(self, upload_id, grid_type=None):
             grids = compiler.process_datas()
             messages.append(compiler.get_message(grids))
         except Exception as err:
-            logger.exception(u"Error while generating measures")
+            logger.exception("Error while generating measures")
             transaction.abort()
             send_error(
                 self.request,
@@ -664,7 +704,7 @@ def compile_measures_task(self, upload_id, grid_type=None):
             )
             return False
         else:
-            logger.info(u"{0} measure grids were handled".format(len(grids)))
+            logger.info("{0} measure grids were handled".format(len(grids)))
             send_success(
                 self.request,
                 mail_addresses,
@@ -672,11 +712,83 @@ def compile_measures_task(self, upload_id, grid_type=None):
                 compiler.get_message(grids),
             )
             logger.info(
-                u"A success email has been sent to {0}".format(
+                "A success email has been sent to {0}".format(
                     mail_addresses
                 )
             )
 
     transaction.commit()
-    logger.info(u"The transaction has been commited")
-    logger.info(u"* Task SUCCEEDED !!!")
+    logger.info("The transaction has been commited")
+    logger.info("* Task SUCCEEDED !!!")
+
+
+@celery_app.task(bind=True)
+def scheduled_compile_measure_task(self, force=False):
+    """
+    Scheduled Celery task to handle automatic measures compilation
+    for Synchronized Datas
+
+    ACTIVATE THIS TASK IN CELERY'S INI CONFIG FILE IF NEEDED
+    Eg:
+        [celerybeat:accounting_measure_compute]
+        task = endi_celery.tasks.accounting_measure_compute.scheduled_compile_measure_task
+        type = crontab
+        schedule = {"minute": 0, "hour": 6}
+    """
+    logger.info(
+        "Launching scheduled compile measure task for synchronized datas"
+    )
+    yesterday = datetime.date.today() - datetime.timedelta(1)
+    upload = AccountingOperationUpload.query().filter(
+        AccountingOperationUpload.filetype ==
+        AccountingOperationUpload.SYNCHRONIZED_ACCOUNTING
+    ).filter(
+        AccountingOperationUpload.created_at >= yesterday
+    ).order_by(
+        desc(AccountingOperationUpload.id)
+    ).first()
+    if upload is None:
+        logger.info(
+            "ABORT : no upload with 'synchronized_accounting' type \
+since yesterday"
+        )
+    else:
+        transaction.begin()
+        logger.info(
+            "Compiling the measures for upload {} ...".format(upload.id)
+        )
+        upload_object = AccountingOperationUpload.get(upload.id)
+        measure_compiler_factories = [
+            TreasuryMeasureCompiler,
+            IncomeStatementMeasureCompiler,
+        ]
+        for measure_compiler_factory in measure_compiler_factories:
+            try:
+                measure_compiler = measure_compiler_factory(
+                    upload_object, upload_object.operations
+                )
+                measure_compiler.process_datas()
+            except Exception as err:
+                transaction.abort()
+                logger.exception("ERROR : {}".format(err.message))
+                mail_addresses = get_recipients_addresses(self.request)
+                if mail_addresses:
+                    send_error(
+                        self.request,
+                        mail_addresses,
+                        measure_compiler.label,
+                        err.message
+                    )
+                    logger.error(
+                        "An error mail has been sent to {0}".format(
+                            mail_addresses
+                        )
+                    )
+                logger.exception(
+                    "Transaction has been rollbacked"
+                )
+                return False
+        transaction.commit()
+        logger.info(
+            "SUCCESS : Measure computing transaction has been commited"
+        )
